@@ -14,9 +14,9 @@
  */
 
 static const char my_name[] = "shc";
-static const char version[] = "Version 3.8.6";
+static const char version[] = "Version 3.8.7";
 static const char subject[] = "Generic Script Compiler";
-static const char cpright[] = "Copyright (c) 1994-2006";
+static const char cpright[] = "Copyright (c) 1994-2009";
 static const struct { const char * f, * s, * e; }
 	author = { "Francisco", "Rosales", "<frosal@fi.upm.es>" };
 
@@ -102,7 +102,7 @@ static const char * help[] = {
 #define SIZE 4096
 
 static char * file;
-static time_t date[1];
+static char   date[21];
 static char * mail = "Please contact your provider";
 static char   rlax[1];
 static char * shll;
@@ -238,14 +238,14 @@ static const char * RTC[] = {
 "int chkenv(int argc)",
 "{",
 "	char buff[512];",
-"	unsigned mask, m;",
+"	unsigned long mask, m;",
 "	int l, a, c;",
 "	char * string;",
 "	extern char ** environ;",
 "",
-"	mask  = (unsigned)chkenv;",
-"	mask ^= (unsigned)getpid() * ~mask;",
-"	sprintf(buff, \"x%x\", mask);",
+"	mask  = (unsigned long)&chkenv;",
+"	mask ^= (unsigned long)getpid() * ~mask;",
+"	sprintf(buff, \"x%lx\", mask);",
 "	string = getenv(buff);",
 "#if DEBUGEXEC",
 "	fprintf(stderr, \"getenv(%s)=%s\\n\", buff, string ? string : \"<null>\");",
@@ -253,11 +253,11 @@ static const char * RTC[] = {
 "	l = strlen(buff);",
 "	if (!string) {",
 "		/* 1st */",
-"		sprintf(&buff[l], \"=%u %d\", mask, argc);",
+"		sprintf(&buff[l], \"=%lu %d\", mask, argc);",
 "		putenv(strdup(buff));",
 "		return 0;",
 "	}",
-"	c = sscanf(string, \"%u %d%c\", &m, &a, buff);",
+"	c = sscanf(string, \"%lu %d%c\", &m, &a, buff);",
 "	if (c == 2 && m == mask) {",
 "		/* 3rd */",
 "		rmarg(environ, &string[-l - 1]);",
@@ -328,7 +328,7 @@ static const char * RTC[] = {
 "	 key(pswd, pswd_z);",
 "	arc4(msg1, msg1_z);",
 "	arc4(date, date_z);",
-"	if (date[0] && date[0]<time(NULL))",
+"	if (date[0] && (atoll(date)<time(NULL)))",
 "		return msg1;",
 "	arc4(shll, shll_z);",
 "	arc4(inlo, inlo_z);",
@@ -420,6 +420,7 @@ static int parse_an_arg(int argc, char * argv[])
 	extern char * optarg;
 	const char * opts = "e:m:f:i:x:l:rvDTCAh";
 	struct tm tmp[1];
+	time_t expdate;
 	int cnt, l;
 	char ctrl;
 
@@ -431,13 +432,17 @@ static int parse_an_arg(int argc, char * argv[])
 		if (cnt == 3) {
 			tmp->tm_mon--;
 			tmp->tm_year -= 1900;
-			date[0] = mktime(tmp);
+			expdate = mktime(tmp);
 		}
-		if (cnt != 3 || date[0] <= 0) {
+		if (cnt != 3 || expdate <= 0) {
 			fprintf(stderr, "%s parse(-e %s): Not a valid value\n",
 				my_name,  optarg);
 			return -1;
 		}
+		sprintf(date, "%lld", (long long)expdate);
+		if (verbose) fprintf(stderr, "%s -e %s", my_name, ctime(&expdate));
+		expdate = atoll(date);
+		if (verbose) fprintf(stderr, "%s -e %s", my_name, ctime(&expdate));
 		break;
 	case 'm':
 		mail = optarg;
@@ -819,7 +824,7 @@ int write_C(char * file, char * argv[])
 	int pswd_z = sizeof(pswd);
 	char* msg1 = strdup("has expired!\n");
 	int msg1_z = strlen(msg1) + 1;
-	int date_z = sizeof(date);
+	int date_z = strlen(date) + 1;
 	char* kwsh = strdup(shll);
 	int shll_z = strlen(shll) + 1;
 	int inlo_z = strlen(inlo) + 1;
@@ -896,7 +901,7 @@ int write_C(char * file, char * argv[])
 			switch (indx) {
 			case  0: if (pswd_z>=0) {prnt_array(o, pswd, "pswd", pswd_z, 0); pswd_z=done=-1; break;}
 			case  1: if (msg1_z>=0) {prnt_array(o, msg1, "msg1", msg1_z, 0); msg1_z=done=-1; break;}
-			case  2: if (date_z>=0) {prnt_array(o, date, "date", date_z, "(time_t*)"); date_z=done=-1; break;}
+			case  2: if (date_z>=0) {prnt_array(o, date, "date", date_z, 0); date_z=done=-1; break;}
 			case  3: if (shll_z>=0) {prnt_array(o, shll, "shll", shll_z, 0); shll_z=done=-1; break;}
 			case  4: if (inlo_z>=0) {prnt_array(o, inlo, "inlo", inlo_z, 0); inlo_z=done=-1; break;}
 			case  5: if (xecc_z>=0) {prnt_array(o, xecc, "xecc", xecc_z, 0); xecc_z=done=-1; break;}
